@@ -2,6 +2,11 @@
 ## Many of the following are simple wrappers for C functions, used largely 
 ## for testing purposes
 
+mgcv.omp <- function() {
+## does open MP appear to be available?
+  oo <- .C(C_mgcv_omp,a=as.integer(-1))
+  if (oo$a==1) TRUE else FALSE
+}
 
 mvn.ll <- function(y,X,beta,dbeta=NULL) {
 ## to facilitate testing of MVN routine mvn_ll.
@@ -49,10 +54,10 @@ pinv <- function(X,svd=FALSE) {
   X
 } ## end pinv
 
-pqr2 <- function(x,nt=1) {
+pqr2 <- function(x,nt=1,nb=30) {
 ## Function for parallel pivoted qr decomposition of a matrix using LAPACK
-## householder routines...
-## library(mgcv); n <- 10000;p<-1000;x <- matrix(runif(n*p),n,p)
+## householder routines. Currently uses a block algorithm.
+## library(mgcv); n <- 4000;p<-3000;x <- matrix(runif(n*p),n,p)
 ## system.time(qrx <- qr(x,LAPACK=TRUE))
 ## system.time(qrx2 <- mgcv:::pqr2(x,2)) 
 ## system.time(qrx3 <- mgcv:::pqr(x,2)) 
@@ -60,12 +65,54 @@ pqr2 <- function(x,nt=1) {
   p <- ncol(x)
   beta <- rep(0.0,p)
   piv <- as.integer(rep(0,p))
-  xc <- x*1
-  rank <- .Call(C_mgcv_Rpiqr,xc,beta,piv,nt)
-  ret <- list(qr=xc,rank=rank,qraux=beta,pivot=piv+1)
+  ## need to force a copy of x, otherwise x will be over-written 
+  ## by .Call *in environment from which function is called*
+  x <- x*1  
+  rank <- .Call(C_mgcv_Rpiqr,x,beta,piv,nt,nb)
+  ret <- list(qr=x,rank=rank,qraux=beta,pivot=piv+1)
   attr(ret,"useLAPACK") <- TRUE
   class(ret) <- "qr"
   ret
+} ## pqr2
+
+pbsi <- function(R,nt=1,copy=TRUE) {
+## parallel back substitution inversion of upper triangular R
+## library(mgcv); n <- 5000;p<-4000;x <- matrix(runif(n*p),n,p)
+## qrx <- mgcv:::pqr2(x,2);R <- qr.R(qrx)
+## system.time(Ri <- mgcv:::pbsi(R,2))
+## system.time(Ri2 <- backsolve(R,diag(p)));range(Ri-Ri2)
+  if (copy) R <- R * 1 ## ensure that R modified only within pbsi
+ .Call(C_mgcv_Rpbsi,R,nt)
+ R
+} ## pbsi
+
+pchol <- function(A,nt=1,nb=30) {
+## parallel Choleski factorization.
+## library(mgcv);
+## set.seed(2);n <- 200;r <- 190;A <- tcrossprod(matrix(runif(n*r),n,r))
+## system.time(R <- chol(A,pivot=TRUE));system.time(L <- mgcv:::pchol(A));range(R[1:r,]-L[1:r,])
+## system.time(L <- mgcv:::pchol(A,nt=2,nb=30))
+## piv <- attr(L,"pivot");attr(L,"rank");range(crossprod(L)-A[piv,piv])
+## should nb be obtained from 'ILAENV' as page 23 of Lucas 2004??
+  piv <- as.integer(rep(0,ncol(A)))
+  A <- A*1 ## otherwise over-write in calling env!
+  rank <- .Call(C_mgcv_Rpchol,A,piv,nt,nb)
+  attr(A,"pivot") <- piv+1;attr(A,"rank") <- rank
+  A
+}
+
+pRRt <- function(R,nt=1) {
+## parallel RR' for upper triangular R
+## following creates index of lower triangular elements...
+## n <- 4000;a <- rep(1:n,n);b <- rep(1:n,each=n)-1;which(a-b>0) -> ii;a[ii]+b[ii]*n->ii
+## library(mgcv);R <- matrix(0,n,n);R[ii] <- runif(n*(n+1)/2)
+## Note: A[a-b<=0] <- 0 zeroes upper triangle 
+## system.time(A <- mgcv:::pRRt(R,2))
+## system.time(A2 <- tcrossprod(R));range(A-A2)
+  n <- nrow(R)
+  A <- matrix(0,n,n)
+  .Call(C_mgcv_RPPt,A,R,nt)
+  A
 }
 
 block.reorder <- function(x,n.blocks=1,reverse=FALSE) {
@@ -82,7 +129,8 @@ block.reorder <- function(x,n.blocks=1,reverse=FALSE) {
  oo <- .C(C_row_block_reorder,x=as.double(x),as.integer(r),as.integer(cols),
           as.integer(nb),as.integer(reverse));
  matrix(oo$x,r,cols)
-}
+} ## block.reorder
+
 
 pqr <- function(x,nt=1) {
 ## parallel QR decomposition, using openMP in C, and up to nt threads (only if worthwhile)
